#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <unistd.h>
#include <sys/stat.h>
#include <string.h>

#include "options.h"
#include "choices.h"
#include "rox_filer_action.h"
#include "menu_images.h"
#include "utils.h"

#include "Icons/group.xpm"
#include "Icons/null.xpm"
#include "Icons/file.xpm"

#include "rox/gtkicontheme.h"

GtkWidget *get_mime_type_image(const char *media, const char* subtype);
GtkWidget *get_icon_by_file_path(const char *file);
#if 0
static gchar *get_rox_filer_themename(void);
static void set_icon_theme(void);
static gchar *check_for_globicon(const char *file);
GdkPixbuf *scale_pixbuf(GdkPixbuf *src, int max_w, int max_h, int uncond);
GdkPixbuf *create_pixbuf(char *path, char ** default_icon, 
			gint width, gint height, gint unconditional);
GtkWidget *create_pixmap(char *path, char ** default_icon,
			gint width, gint height, gint unconditional);
GtkWidget *get_icon_by_file_path(const char *file);
GtkWidget *get_menu_pixmap_path (gchar *path);
#endif

static RoxIconTheme *icon_theme = NULL;


//~ try to find the name of the option by parsing the ROX-Filer config
/* This function adapted from the options.c file in ROX-Filer 2.1.2
   by Thomas Leonard. */
static gchar *get_rox_filer_themename(void)
{
	guchar *path;
	xmlDoc	*doc;
	gchar *value;
	value = NULL;
	path = choices_find_path_load("Options", "ROX-Filer");
	if (path)
	{
		doc = xmlParseFile(path);
		if (doc)
		{
			xmlNode *root, *node;
			root = xmlDocGetRootElement(doc);
			g_return_if_fail(strcmp(root->name, "Options") == 0);
			for (node = root->xmlChildrenNode; node; node = node->next)
			{
				if (node->type != XML_ELEMENT_NODE)
					continue;
				if (strcmp(node->name, "Option") != 0)
					continue;
				if (strcmp(xmlGetProp(node, "name"), "icon_theme") == 0) {
					value = xmlNodeGetContent(node);
					break;
				}
			}
			xmlFreeDoc(doc);
		}
	}
	return value;
}

/* This function adapted from the type.c file in ROX-Filer 2.1.2
   by Thomas Leonard. */
static void set_icon_theme(void)
{
	extern Option entry_height;
	extern Option theme_name;
	RoxIconInfo *info;
	char *theme;
	
	theme = theme_name.value;
	if (!theme || !*theme) theme = get_rox_filer_themename();
	
	extern RoxIconTheme *icon_theme;
	if (!icon_theme) icon_theme = rox_icon_theme_new();
	if (!theme || !*theme) theme = "ROX";

	while (1)
	{
		rox_icon_theme_set_custom_theme(icon_theme, theme);
		info = rox_icon_theme_lookup_icon(icon_theme,
				"mime-application:postscript",
				entry_height.int_value, 0);
		if (!info)
		{
			info = rox_icon_theme_lookup_icon(icon_theme,
					"gnome-mime-application-postscript",
					entry_height.int_value, 0);
		}
		if (info)
		{
			rox_icon_info_free(info);
			return;
		}

		if (strcmp(theme, "ROX") == 0)
			break;

		
		theme = "ROX";
	}

	rox_icon_theme_rescan_if_needed(icon_theme);
	exit(1);
}

/* This function adapted from the type.c file in ROX-Filer 2.1.2
   by Thomas Leonard. */
GtkWidget *get_mime_type_image(const char *media, 
                                       const char* subtype) {
					       
	char	*type_name, *path;
	extern Option entry_width;
	extern Option entry_height;
	extern Option scale_up;
	GdkPixbuf *full;
	GtkWidget *image;
	image = NULL;
					       
	type_name = g_strconcat(media, "_", subtype,".png", NULL);			       
	path = choices_find_path_load(type_name, "MIME-icons");
	g_free(type_name);
	if (path) {
			image = create_pixmap(path, NULL, entry_width.int_value, 
			                   entry_height.int_value, scale_up.int_value);
			g_free(path);
			return image;
	}
	
	g_free(path);
	type_name = g_strconcat("mime-", media, ":",subtype, NULL);
	
	
	set_icon_theme();
	
	full = rox_icon_theme_load_icon(icon_theme, type_name, entry_height.int_value, 0, NULL);
	g_free(type_name);
	
	if (!full)
	{
		/* Ugly hack... try for a GNOME icon */
		type_name = g_strconcat("gnome-mime-", media, "-", subtype, NULL);
		full = rox_icon_theme_load_icon(icon_theme,
						type_name,
						entry_height.int_value, 0, NULL);
		g_free(type_name);
	}
	if (!full)
	{
		/* Try for a media type */
		type_name = g_strconcat("mime-", media, NULL);
		full = rox_icon_theme_load_icon(icon_theme,
						type_name,
						entry_height.int_value, 0, NULL);
		g_free(type_name);
	}
	if (full)
	{
		image = gtk_image_new_from_pixbuf(full);
		g_object_unref(full);
	}
	return image;
}
					       
/* This function adapted from the usericons.c file in ROX-Filer 2.1.2
   by Thomas Leonard. */
static gchar *check_for_globicon(const char *file)
{
	guchar *path, *globicon_path;
	globicon_path = NULL;
	xmlDocPtr doc;

	path = choices_find_path_load("globicons", "ROX-Filer");
	if (!path)
		return;	/* Nothing to load */

	doc = xmlParseFile(path);
	if (doc)
	{
		xmlNodePtr node, icon, root;
		char	   *match;
		
		root = xmlDocGetRootElement(doc);
		
		/* Handle the new XML file format */
		for (node = root->xmlChildrenNode; node; node = node->next)
		{
			gchar *path, *icon_path;

			if (node->type != XML_ELEMENT_NODE)
				continue;
			if (strcmp(node->name, "rule") != 0)
				continue;
			icon = get_subnode(node, NULL, "icon");
			if (!icon)
				continue;
			match = xmlGetProp(node, "match");
			if (!match)
				continue;

			icon_path = xmlNodeGetContent(icon);
			
			if (strcmp(file, match) == 0)
			{
				globicon_path = strdup(icon_path);
				g_free(match);
				break;
			}
			g_free(match);
		}
		xmlFreeDoc(doc);
	}
	return globicon_path;
}

/* This function adapted from the pixmaps.c file in ROX-Filer 2.0.1
   by Thomas Leonard. */
GdkPixbuf *scale_pixbuf(GdkPixbuf *src, int max_w, int max_h, int uncond)
{
	int	w, h;
	w = gdk_pixbuf_get_width(src);
	h = gdk_pixbuf_get_height(src);

	if (w <= max_w && h <= max_h && !uncond)
	{
		gdk_pixbuf_ref(src);
		return src;
	}
	else
	{
		float scale_x = ((float) w) / max_w;
		float scale_y = ((float) h) / max_h;
		float scale = MAX(scale_x, scale_y);
		int dest_w = w / scale;
		int dest_h = h / scale;
		
		return gdk_pixbuf_scale_simple(src,
						MAX(dest_w, 1),
						MAX(dest_h, 1),
						GDK_INTERP_BILINEAR);
	}
}

GdkPixbuf *create_pixbuf(char *path, char ** default_icon, 
                         gint width, gint height, gint unconditional)
{
   GdkPixbuf *pixbuf, *pixbuf_scaled;

	g_return_val_if_fail(path != NULL, NULL);

   if (!access(path, R_OK)) {
		pixbuf = gdk_pixbuf_new_from_file(path, NULL);
   } else {
		pixbuf = gdk_pixbuf_new_from_xpm_data((const char **) default_icon);
   }
	g_return_val_if_fail(pixbuf != NULL, NULL);
	pixbuf_scaled = scale_pixbuf(pixbuf, width, height, unconditional);
	
	/*if (GDK_IS_PIXBUF(pixbuf_scaled) && pixbuf != pixbuf_scaled) g_object_unref(pixbuf);*/
	g_object_unref(pixbuf);

	return pixbuf_scaled;
}

GtkWidget *create_pixmap(char *path, char ** default_icon, 
                         gint width, gint height, gint unconditional)
{
   GtkWidget *pixmap;
	GdkPixbuf *pixbuf;
	
	pixbuf = create_pixbuf(path, default_icon, width, height, unconditional);
	g_return_val_if_fail(pixbuf != NULL, NULL);
	pixmap = gtk_image_new_from_pixbuf(GDK_PIXBUF(pixbuf));
	
	/*if (GDK_IS_PIXBUF(pixbuf) ) g_object_unref(pixbuf);*/
	g_object_unref(pixbuf);

	return pixmap;
	
}

GtkWidget *get_icon_by_file_path(const char *file) {
	extern Option entry_width;
	extern Option entry_height;
	extern Option scale_up;
	char *media, *subtype, *fulltype;
	GtkWidget *image;
	gchar *path;

	//~ check if there is any special icon choosen for this file
	path = check_for_globicon(file);
	if (path) {
		image = create_pixmap(path, NULL, entry_width.int_value, 
			                   entry_height.int_value, scale_up.int_value);
		return image;
	}
	
	fulltype = rox_filer_file_type(file);

	media=fulltype;
	subtype = strstr(fulltype, G_DIR_SEPARATOR_S);
	if (subtype) {
		*subtype = 0;
		subtype = subtype+1;
	}

	image = get_mime_type_image(media, subtype);
	g_free(fulltype);
	return image;
}

/* Takes a path and returns the GtkPixmap widget that will be used
   for the menu item. */
GtkWidget *get_menu_pixmap_path (gchar *path) {
	
	extern Option group_image;
	extern Option entry_image;
	extern Option file_image;
	extern Option entry_width;
	extern Option entry_height;
	extern Option scale_up;
	
	extern GdkPixbuf *default_group_pixmap;
	extern GdkPixbuf *default_entry_pixmap;
	extern GdkPixbuf *default_file_pixmap;
	
	gchar *icon_path;
	GtkWidget *ret;
	
	icon_path = g_build_path(G_DIR_SEPARATOR_S, path, ".DirIcon", NULL);
	
	if (is_appdir(path)) {
		
		if (!access((char*)icon_path, R_OK)) {
			ret = create_pixmap(icon_path, NULL, entry_width.int_value, 
			                   entry_height.int_value, scale_up.int_value);
		} else {
			/* No .DirIcon, check for AppIcon.xpm. */
			g_free(icon_path);
			icon_path = g_build_path(G_DIR_SEPARATOR_S, path, "AppIcon.xpm", NULL);
			if (!access((char*)icon_path, R_OK)) {
				ret = create_pixmap(icon_path, NULL, entry_width.int_value, 
			                   entry_height.int_value, scale_up.int_value);
			} else {
				/* No icon file.  Use icon from theme. */
				/* No icon file.  Use default. */
				ret = get_mime_type_image("application","executable"); 
				if (!ret){ 
					if (!default_entry_pixmap) {
					default_entry_pixmap = create_pixbuf(entry_image.value, 
						null_xpm, entry_width.int_value, 
					        entry_height.int_value, scale_up.int_value);
					}
					ret = gtk_image_new_from_pixbuf(GDK_PIXBUF(default_entry_pixmap));
				}
			}
		}
		
	} else if (is_regdir(path)) {
		if (!access((char*)icon_path, R_OK)) {
			ret = create_pixmap(icon_path, NULL, entry_width.int_value, 
			                   entry_height.int_value, scale_up.int_value);
		} else {
			/* No .DirIcon, so clear icon_path. */
			ret = get_mime_type_image("inode","directory"); 
			if (!ret){
				if (!default_group_pixmap) {
					default_group_pixmap = create_pixbuf(group_image.value, 
						group_xpm, entry_width.int_value,
						entry_height.int_value, scale_up.int_value);
				}
				ret = gtk_image_new_from_pixbuf(GDK_PIXBUF(default_group_pixmap));
			}
		}
		
	} else { /* Path is a regular file. */
		//~ g_free(icon_path);
		ret = get_icon_by_file_path(path);
		if (!ret){
			if (!default_file_pixmap) {
				default_file_pixmap = create_pixbuf(file_image.value,
					file_xpm, entry_width.int_value,
					entry_height.int_value, scale_up.int_value);
				}
			ret = gtk_image_new_from_pixbuf(GDK_PIXBUF(default_file_pixmap));
		}
		
	}
	g_free(icon_path);
	gtk_widget_show(ret);
	return ret;
}
